@@ -23,10 +23,10 @@ sub _time_func ($loop) {
     return (time, times) »-« @s;
 }
 
-sub timethese ($count, $hash) is export {
-    for %$hash.kv -> $name, $code {
+sub timethese ($count, %hash) is export {
+    for %hash.kv -> $name, $code {
         my @time = timeit($count, $code);
-        say "$name: { @time[0] != 0 ?? $count / @time[0] :: "Inf" } / s";
+        say "$name: { @time[0] != 0 ?? int($count / @time[0]) :: "Inf" }/s";
     }
 }
 
