@@ -3,8 +3,8 @@ use v6;
 
 module t_LKT_B_L_Fre;
 
-my $xy = 'BF'; # is constant
-my Str %text_strings = ( # is constant
+my $xy is constant = 'BF';
+my Str %text_strings is constant = (
 	'two' => $xy~' - sky pie rye',
 	'three' => $xy~' - eat {knife}',
 );
