@@ -148,7 +148,7 @@ sub warn_cygwin {
 
 sub assert_ghc {
     my $self = shift;
-    my $ghc = $self->can_run($ENV{GHC} || 'ghc');
+    my $ghc = $self->can_run($ENV{GHC} || ( 'ghc' . $Config{_exe} ) );
     my $ghcver = `$ghc --version`;
     ($ghcver =~ /Glasgow.*\bversion\s*(\S+)/s) or die << '.';
 *** Cannot find a runnable 'ghc' from path.
