@@ -1,12 +1,12 @@
 {-# OPTIONS_GHC -fglasgow-exts -fth -cpp #-}
 
-{-
+{-|
     Parrot IMC implementation.
 
-    All that is gold does not glitter,
-    Not all those who wander are lost;
-    The old that is strong does not wither,
-    Deep roots are not reached by the frost.
+>   All that is gold does not glitter,
+>   Not all those who wander are lost;
+>   The old that is strong does not wither,
+>   Deep roots are not reached by the frost.
 -}
 
 module IMC (
